<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Fields.php 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for fields table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Fields extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'fields';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Private fields
     * 
     * @var array
     */
    protected $_privateFields = array(
        'gid' , 
        'the_geom'
    );

    /**
     * Get fields
     * 
     * @param array $options Available options:
     * <ul><li>whereStm string - Fetch rows that match the statement</li>
     * <li>whereLayerId integer - Fetch rows that match the layer id</li>
     * <li>asArray bool - Get results as array</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'layer_id' , 
            'name' , 
            'expression' , 
            'visibility' ,
            'escape' ,
            'order'
        );
        
        $select->from($this, $fields);
        
        if (isset($options['whereStm'])) {
            $select->where($options['whereStm']);
        }
        
        if (isset($options['whereLayerId'])) {
            $select->where($this->_name . '.layer_id = ?', $options['whereLayerId']);
        }
        
        $this->whereId($select, $options);
        
        $select->order($this->_name . '.order asc');
        
        $result = $this->fetchAll($select);
        
        //Convert to array
        if (isset($options['asArray']) && $options['asArray'] == true) {
            $result = $result->toArray();
        }
        
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        //Get column
        $column = $data->column;
        
        if ($column == 'name') {
            $layerSettingsTable = new Giscuit_Db_Table_LayerSettings();
            $layerDefaultsTable = new Giscuit_Db_Table_LayerDefaults();
            
            $layerSettingsTable->cleanupLabelFields($row->$column, $data->data);
            $layerDefaultsTable->cleanupLabelFields($row->$column, $data->data);
        }

        if($column == 'expression') {
            $row->$column = $data->getUnescaped('data');
        } else {
            $row->$column = $data->data;
        }
        
        return $row->save();
    
    }

    /**
     * Add item
     * 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return Zend_Db_Table_Row
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        $newRow->layer_id = $data->layer_id;
        
        $id = $newRow->save();
        
        $newRow->order = $id;
        $newRow->save();
        
        return $newRow;
    }

    /**
     * Add initial fields of the given geometry
     * 
     * @param mixed $table Table settings
     * @return Zend_Db_Table_Row
     */
    public function addInitial ($table)
    {
        $defaultField = '';
        
        $settings = $table->getSettings();
        $layerId = $settings['id'];
        
        //Remove current fields related to this layer id they exist
        $delWhere = $this->getAdapter()->quoteInto($this->_name . '.layer_id = ?', $layerId);
        $this->delete($delWhere);
        
        $cols = $table->info('cols');
        
        foreach ($cols as $col) {
            
            if (in_array($col, $this->_privateFields)) {
                continue;
            }
            
            if(empty($defaultField)) {
                $defaultField = $col;
            }
            
            $newRow = $this->createRow();
            $newRow->layer_id = $layerId;
            $newRow->name = $col;
            $pkey = $newRow->save();
            
            $newRow->order = $pkey;
            $newRow->save();
        }
        
        return $defaultField;
    }
    
    /**
     * Check if field belongs to current user
     * 
     * @param integer $id Field id
     * @return void
     */
    public function checkCurrUserOwnField($id)
    {
        $rows = $this->getItems(array(
            'whereId' => $id
        ))->toArray();
        if(count($rows) == 0) {
            throw new Exception("Invalid field id ({$id})");
        }
        
        $layersTable = new Giscuit_Db_Table_Layers();
        $layersTable->checkCurrUserOwnLayer($rows[0]['layer_id']);
    }
}
